use structopt::StructOpt;
use std::{
    boxed::Box,
    error::Error,
    net::{
        IpAddr,
        Ipv4Addr,
    },
};
use crate::{cli::Options, config::Config};

mod cli;
mod config;
mod net;
#[cfg(feature = "sqlite")]
mod sqlite;
mod graphql;

const DEFAULT_CONFIG_PATH: &'static str = "/etc/noise-server/config.toml";
const DEFAULT_DATABASE_PATH: &'static str = "/var/lib/noise-server/noise-server.sqlite";
const DEFAULT_IP_ADDRESS: IpAddr = IpAddr::V4(Ipv4Addr::UNSPECIFIED);
const DEFAULT_IP_ADDRESS_STR: &'static str = "0.0.0.0";
#[cfg(debug_assertions)]
const DEFAULT_PORT: u16 = 8080;
#[cfg(debug_assertions)]
const DEFAULT_PORT_STR: &'static str = "8080";
#[cfg(not(debug_assertions))]
const DEFAULT_PORT: u16 = 80;
#[cfg(not(debug_assertions))]
const DEFAULT_PORT_STR: &'static str = "80";

#[tokio::main]
async fn main() -> Result<(), Box<dyn Error>> {
    let (config, options) = Config::get_with_options(
        Options::clap().get_matches()
    ).await?;

    if options.generate_config {
        println!("{}", toml::to_string_pretty(&config)?);
        return Ok(());
    }

    dbg!(&config);

    #[cfg(feature = "sqlite")]
    let sqlite_connection = sqlite::connect(&config.database_path).await?;

    #[cfg(feature = "sqlite")]
    sqlite::load_template(&sqlite_connection).await?;

    net::web(config.ip_address, config.port, sqlite_connection).await;

    Ok(())    
}