use uuid::Uuid;
use std::path::PathBuf;
use sqlx::sqlite::{
    SqlitePool,
    SqliteConnectOptions,
    SqliteQueryResult,
    SqliteSynchronous,
};

/// Prepare sqlite database connection
pub async fn connect(path: &PathBuf) -> sqlx::Result<SqlitePool> {
    let connect_options = SqliteConnectOptions::new()
        .filename(&path)
        .create_if_missing(true)
        .synchronous(SqliteSynchronous::Normal);
    SqlitePool::connect_with(connect_options).await
}

/// Create tables from template
pub async fn load_template(sqlite_handle: &SqlitePool) -> sqlx::Result<SqliteQueryResult> {
    sqlx::query(concat!(
        include_str!("users.sql"),
        include_str!("security_preferences.sql"),
        include_str!("privacy_preferences.sql"),
        include_str!("notification_preferences.sql"),
        include_str!("external_servers_privacy_preferences.sql"),
        include_str!("external_servers_preferences.sql"),
    )).execute(sqlite_handle).await
}

/// Create message table from template
pub async fn create_chat(sqlite_handle: &SqlitePool, uuid: &Uuid) -> sqlx::Result<SqliteQueryResult> {
    sqlx::query(format!(
        include_str!("msgdata_template.sql"),
        chat_id = uuid.to_simple(),
    ).as_str()).execute(sqlite_handle).await
}